package com.sun.java.swing.plaf.motif;

import java.beans.PropertyChangeEvent;
import java.awt.Color;
import java.awt.Container;
import java.awt.event.KeyListener;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseMotionListener;
import javax.swing.plaf.basic.BasicComboPopup;
import javax.swing.JComboBox;
import java.beans.PropertyChangeListener;
import java.awt.LayoutManager;
import java.awt.Component;
import javax.swing.border.Border;
import java.awt.Rectangle;
import java.awt.Graphics;
import javax.swing.plaf.basic.ComboPopup;
import java.awt.Insets;
import java.awt.Dimension;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.Icon;
import java.io.Serializable;
import javax.swing.plaf.basic.BasicComboBoxUI;

public class MotifComboBoxUI extends BasicComboBoxUI implements Serializable
{
    Icon arrowIcon;
    static final int HORIZ_MARGIN = 3;
    
    public MotifComboBoxUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new MotifComboBoxUI();
    }
    
    @Override
    public void installUI(JComponent c) {
        super.installUI(c);
        this.arrowIcon = new MotifComboBoxUI.MotifComboBoxArrowIcon(UIManager.getColor("controlHighlight"), UIManager.getColor("controlShadow"), UIManager.getColor("control"));
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                if (MotifComboBoxUI.access$000(MotifComboBoxUI.this) != null) {
                    MotifComboBoxUI.access$000(MotifComboBoxUI.this).setBackground(UIManager.getColor("text"));
                }
            }
        });
    }
    
    @Override
    public Dimension getMinimumSize(JComponent component) {
        if (!this.isMinimumSizeDirty) {
            return new Dimension(this.cachedMinimumSize);
        }
        Insets insets = this.getInsets();
        Dimension displaySize = this.getDisplaySize();
        displaySize.height += insets.top + insets.bottom;
        displaySize.width += insets.left + insets.right + this.iconAreaWidth();
        this.cachedMinimumSize.setSize(displaySize.width, displaySize.height);
        this.isMinimumSizeDirty = false;
        return displaySize;
    }
    
    @Override
    protected ComboPopup createPopup() {
        return new MotifComboBoxUI.MotifComboPopup(this.comboBox);
    }
    
    @Override
    protected void installComponents() {
        if (this.comboBox.isEditable()) {
            this.addEditor();
        }
        this.comboBox.add(this.currentValuePane);
    }
    
    @Override
    protected void uninstallComponents() {
        this.removeEditor();
        this.comboBox.removeAll();
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        int hasFocus = this.comboBox.hasFocus();
        if (this.comboBox.isEnabled()) {
            graphics.setColor(this.comboBox.getBackground());
        }
        else {
            graphics.setColor(UIManager.getColor("ComboBox.disabledBackground"));
        }
        graphics.fillRect(0, 0, component.getWidth(), component.getHeight());
        if (!this.comboBox.isEditable()) {
            this.paintCurrentValue(graphics, this.rectangleForCurrentValue(), (boolean)hasFocus);
        }
        Rectangle rectangleForArrowIcon = this.rectangleForArrowIcon();
        this.arrowIcon.paintIcon(component, graphics, rectangleForArrowIcon.x, rectangleForArrowIcon.y);
        if (!this.comboBox.isEditable()) {
            Border border = this.comboBox.getBorder();
            Insets borderInsets;
            if (border != null) {
                borderInsets = border.getBorderInsets(this.comboBox);
            }
            else {
                borderInsets = new Insets(0, 0, 0, 0);
            }
            if (MotifGraphicsUtils.isLeftToRight(this.comboBox)) {
                rectangleForArrowIcon.x -= 5;
            }
            else {
                rectangleForArrowIcon.x += rectangleForArrowIcon.width + 3 + 1;
            }
            rectangleForArrowIcon.y = borderInsets.top;
            rectangleForArrowIcon.width = 1;
            rectangleForArrowIcon.height = this.comboBox.getBounds().height - borderInsets.bottom - borderInsets.top;
            graphics.setColor(UIManager.getColor("controlShadow"));
            graphics.fillRect(rectangleForArrowIcon.x, rectangleForArrowIcon.y, rectangleForArrowIcon.width, rectangleForArrowIcon.height);
            ++rectangleForArrowIcon.x;
            graphics.setColor(UIManager.getColor("controlHighlight"));
            graphics.fillRect(rectangleForArrowIcon.x, rectangleForArrowIcon.y, rectangleForArrowIcon.width, rectangleForArrowIcon.height);
        }
    }
    
    @Override
    public void paintCurrentValue(Graphics g, Rectangle rectangle, boolean b) {
        Component listCellRendererComponent = this.comboBox.getRenderer().getListCellRendererComponent(this.listBox, this.comboBox.getSelectedItem(), -1, false, false);
        listCellRendererComponent.setFont(this.comboBox.getFont());
        if (this.comboBox.isEnabled()) {
            listCellRendererComponent.setForeground(this.comboBox.getForeground());
            listCellRendererComponent.setBackground(this.comboBox.getBackground());
        }
        else {
            listCellRendererComponent.setForeground(UIManager.getColor("ComboBox.disabledForeground"));
            listCellRendererComponent.setBackground(UIManager.getColor("ComboBox.disabledBackground"));
        }
        this.currentValuePane.paintComponent(g, listCellRendererComponent, this.comboBox, rectangle.x, rectangle.y, rectangle.width, listCellRendererComponent.getPreferredSize().height);
    }
    
    protected Rectangle rectangleForArrowIcon() {
        Rectangle bounds = this.comboBox.getBounds();
        Border border = this.comboBox.getBorder();
        Insets borderInsets;
        if (border != null) {
            borderInsets = border.getBorderInsets(this.comboBox);
        }
        else {
            borderInsets = new Insets(0, 0, 0, 0);
        }
        bounds.x = borderInsets.left;
        bounds.y = borderInsets.top;
        bounds.width -= borderInsets.left + borderInsets.right;
        bounds.height -= borderInsets.top + borderInsets.bottom;
        if (MotifGraphicsUtils.isLeftToRight(this.comboBox)) {
            bounds.x = bounds.x + bounds.width - 3 - this.arrowIcon.getIconWidth();
        }
        else {
            bounds.x += 3;
        }
        bounds.y += (bounds.height - this.arrowIcon.getIconHeight()) / 2;
        bounds.width = this.arrowIcon.getIconWidth();
        bounds.height = this.arrowIcon.getIconHeight();
        return bounds;
    }
    
    @Override
    protected Rectangle rectangleForCurrentValue() {
        int width = this.comboBox.getWidth();
        int height = this.comboBox.getHeight();
        Insets insets = this.getInsets();
        if (MotifGraphicsUtils.isLeftToRight(this.comboBox)) {
            return new Rectangle(insets.left, insets.top, width - (insets.left + insets.right) - this.iconAreaWidth(), height - (insets.top + insets.bottom));
        }
        return new Rectangle(insets.left + this.iconAreaWidth(), insets.top, width - (insets.left + insets.right) - this.iconAreaWidth(), height - (insets.top + insets.bottom));
    }
    
    public int iconAreaWidth() {
        if (this.comboBox.isEditable()) {
            return this.arrowIcon.getIconWidth() + 6;
        }
        return this.arrowIcon.getIconWidth() + 9 + 2;
    }
    
    public void configureEditor() {
        super.configureEditor();
        this.editor.setBackground(UIManager.getColor("text"));
    }
    
    @Override
    protected LayoutManager createLayoutManager() {
        return new MotifComboBoxUI.ComboBoxLayoutManager();
    }
    
    private Component motifGetEditor() {
        return this.editor;
    }
    
    @Override
    protected PropertyChangeListener createPropertyChangeListener() {
        return new MotifComboBoxUI.MotifPropertyChangeListener();
    }
    
    static Component access$000(MotifComboBoxUI motifComboBoxUI) {
        return motifComboBoxUI.motifGetEditor();
    }
    
    protected class MotifComboPopup extends BasicComboPopup
    {
        public MotifComboPopup(JComboBox combo) {
            super(combo);
        }
        
        public MouseMotionListener createListMouseMotionListener() {
            return new MouseMotionAdapter() {};
        }
        
        public KeyListener createKeyListener() {
            return super.createKeyListener();
        }
        
        protected class InvocationKeyHandler extends BasicComboPopup.InvocationKeyHandler
        {
        }
    }
    
    public class ComboBoxLayoutManager extends BasicComboBoxUI.ComboBoxLayoutManager
    {
        @Override
        public void layoutContainer(Container container) {
            if (MotifComboBoxUI.access$000(MotifComboBoxUI.this) != null) {
                Rectangle rectangleForCurrentValue = MotifComboBoxUI.this.rectangleForCurrentValue();
                ++rectangleForCurrentValue.x;
                ++rectangleForCurrentValue.y;
                --rectangleForCurrentValue.width;
                rectangleForCurrentValue.height -= 2;
                MotifComboBoxUI.access$000(MotifComboBoxUI.this).setBounds(rectangleForCurrentValue);
            }
        }
    }
    
    static class MotifComboBoxArrowIcon implements Icon, Serializable
    {
        private Color lightShadow;
        private Color darkShadow;
        private Color fill;
        
        public MotifComboBoxArrowIcon(Color lightShadow, Color darkShadow, Color fill) {
            this.lightShadow = lightShadow;
            this.darkShadow = darkShadow;
            this.fill = fill;
        }
        
        @Override
        public void paintIcon(Component component, Graphics graphics, int n, int n2) {
            int iconWidth = this.getIconWidth();
            int iconHeight = this.getIconHeight();
            graphics.setColor(this.lightShadow);
            graphics.drawLine(n, n2, n + iconWidth - 1, n2);
            graphics.drawLine(n, n2 + 1, n + iconWidth - 3, n2 + 1);
            graphics.setColor(this.darkShadow);
            graphics.drawLine(n + iconWidth - 2, n2 + 1, n + iconWidth - 1, n2 + 1);
            for (int n3 = n + 1, n4 = n2 + 2, n5 = iconWidth - 6; n4 + 1 < n2 + iconHeight; ++n3, n5 -= 2, n4 += 2) {
                graphics.setColor(this.lightShadow);
                graphics.drawLine(n3, n4, n3 + 1, n4);
                graphics.drawLine(n3, n4 + 1, n3 + 1, n4 + 1);
                if (n5 > 0) {
                    graphics.setColor(this.fill);
                    graphics.drawLine(n3 + 2, n4, n3 + 1 + n5, n4);
                    graphics.drawLine(n3 + 2, n4 + 1, n3 + 1 + n5, n4 + 1);
                }
                graphics.setColor(this.darkShadow);
                graphics.drawLine(n3 + n5 + 2, n4, n3 + n5 + 3, n4);
                graphics.drawLine(n3 + n5 + 2, n4 + 1, n3 + n5 + 3, n4 + 1);
            }
            graphics.setColor(this.darkShadow);
            graphics.drawLine(n + iconWidth / 2, n2 + iconHeight - 1, n + iconWidth / 2, n2 + iconHeight - 1);
        }
        
        @Override
        public int getIconWidth() {
            return 11;
        }
        
        @Override
        public int getIconHeight() {
            return 11;
        }
    }
    
    private class MotifPropertyChangeListener extends BasicComboBoxUI.PropertyChangeHandler
    {
        @Override
        public void propertyChange(PropertyChangeEvent e) {
            super.propertyChange(e);
            if (e.getPropertyName() == "enabled" && MotifComboBoxUI.this.comboBox.isEnabled()) {
                Component access$000 = MotifComboBoxUI.access$000(MotifComboBoxUI.this);
                if (access$000 != null) {
                    access$000.setBackground(UIManager.getColor("text"));
                }
            }
        }
    }
}
